#
# APS-18-ID.py  - Site-specific script for Beamline 18-ID at the APS.
#

print "***** Start of APS-18-ID.py *****"

import MpCa
import threading

#---------------------------------------------------------------------------
#
# Create a simple event type used by the EPICS subscription thread to send
# messages to the main thread.  Inspired by
#
#    http://wiki.wxpython.org/LongRunningTasks
#

global EVT_RESULT_ID
global EVT_RESULT
global ResultEvent

EVT_RESULT_ID = wx.NewId()

def EVT_RESULT( win, func ):
	global EVT_RESULT_ID

	win.Connect( -1, -1, EVT_RESULT_ID, func )

class ResultEvent( wx.PyEvent ):
	global EVT_RESULT_ID

	def __init__( self, data ):
		wx.PyEvent.__init__( self )
		self.SetEventType( EVT_RESULT_ID )
		self.data = data

#---------------------------------------------------------------------------
#

global PVOUT_SET_VALUE
global PVOUT_CHECK_IDLE

PVOUT_SET_VALUE = 1
PVOUT_CHECK_IDLE = 2

#---------------------------------------------------------------------------
#

def biocat_setup_pv_display( panel, pv_name ):
	global MpCa

	try:
		del panel.value_callback
	except:
		pass

	try:
		del panel.trigger_pv
	except:
		pass

	try:
		old_panel_value = panel.value
	except:
		old_panel_value = None

	try:
		panel.trigger_pv = MpCa.PV( pv_name )

		pv_value = panel.trigger_pv.caget()

		try:
			panel.value.SetLabel( str(pv_value) )
			panel.value.SetForegroundColour( "blue" )
		except:
			#print "PV %s label not set, due to no panel.value" \
			#	% pv_name
			pass
	except:
		panel.trigger_pv = None

		try:
			panel.value.SetLabel( "PV %s NOT FOUND" % pv_name )
			panel.value.Enable(False)
		except:
			#print "PV NOT FOUND not set, due to no panel.value"
			pass

	if ( panel.trigger_pv != None ):
		panel.value_callback = panel.trigger_pv.add_callback(
			MpCa.DBE_VALUE, panel.trigger_callback_handler, panel )
	else:
		panel.value_callback = None

	return

#---------------------------------------------------------------------------
#

def biocat_ad_trigger():
	print "biocat_ad_trigger() invoked."

	current_sequence_panel = \
	_mpad.frame.image_tab.right_panel.sequence_panel.current_sequence_panel

	print "current_sequence_panel =", current_sequence_panel

	sequence_panel_command = current_sequence_panel.command

	print "sequence_panel_command =", sequence_panel_command

	#------

	detector_controls_tab = _mpad.frame.detector_controls_tab
	input_trigger_panel = \
			detector_controls_tab.pv_trigger_panel.input_panel
	output_trigger_panel = \
			detector_controls_tab.pv_trigger_panel.output_panel

	trigger_pv = output_trigger_panel.trigger_pv

	#------

	input_enabled  = input_trigger_panel.checkbox.GetValue()
	output_enabled = output_trigger_panel.checkbox.GetValue()

	#------

	if ( input_enabled ):
		wx.Yield()

	#------

	if ( output_enabled == False ):
		print "biocat_ad_trigger(): trigger delay disabled."
	else:
		trigger_delay = \
			output_trigger_panel.trigger_delay_ctrl.GetValue()

		trigger_delay = float(trigger_delay)

		#------

		trigger_value = \
			output_trigger_panel.trigger_value_ctrl.GetValue()

		trigger_value = int(trigger_value)

		#------

		if ( trigger_pv == None ):
			print \
	"WARNING: Skipping output PV trigger since the PV is not connected!"
		else:
			print \
		"biocat_ad_trigger(): Sending the output trigger value %d" \
							% trigger_value

			MpCa.poll()
			trigger_pv.caput( trigger_value )
			MpCa.poll()

		#------

		print "biocat_ad_trigger(): Waiting for %f seconds..." \
						% trigger_delay

		time.sleep( trigger_delay )

		print "biocat_ad_trigger(): Finished wait."

	print "biocat_ad_trigger(): Triggering the area detector!"

	sequence_panel_command( current_sequence_panel )

	if ( output_enabled ):
		wx.Yield()

		if ( trigger_pv != None ):
			#
			# Post an event to the output trigger panel telling
			# it to check to see if the area detector is idle.
			# If it is idle, the output trigger will be reset.
			#

			wx.PostEvent( output_trigger_panel, \
					ResultEvent( PVOUT_CHECK_IDLE ) )

	print "biocat_ad_trigger() complete."

#---------------------------------------------------------------------------

class MpadBiocatInputTriggerPanel( wx.Panel ):
	global pv_input_value_callback
	global pv_input_trigger_name_update
	global biocat_ad_trigger
	global biocat_setup_pv_display
	global EVT_RESULT

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.epics_initialized = False

		self.trigger_pv = None
		self.trigger_callback = None

		self.grid = wx.GridBagSizer( hgap=5, vgap=5 )
		self.SetSizer( self.grid )

		row = 0

		#------

		self.entry_label = wx.StaticText( self, -1, "PV Name:" )

		self.grid.Add( self.entry_label, pos=(row,0), \
			flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		self.trigger_row = row + 1
		self.trigger_callback_handler = pv_input_value_callback

		pv_name_record = "pvin_trig_name.value"

		self.pv_name = MpWx.ValueEntry( self,
				_mpad.server_record, pv_name_record,
				size=(200,-1),
				function=pv_input_trigger_name_update )

		self.grid.Add( self.pv_name, pos=(row,1) )

		#------

		self.checkbox = wx.CheckBox( self, -1, "Enable" )

		self.grid.Add( self.checkbox, pos=(row,2) )

		row = row + 1

		#------

		self.value_label = wx.StaticText( self, -1, "PV Value:" )

		self.grid.Add( self.value_label, pos=(row,0) )

		#------

		self.value = wx.StaticText( self, -1, "--------" )

		self.grid.Add( self.value, pos=(row,1) )

		trigger_pv_name = self.pv_name.GetValue()

		biocat_setup_pv_display( self, trigger_pv_name )

		row = row + 1

		#------

		self.trigger_value_label = wx.StaticText( self, -1, \
							"PV Trigger Value:" )

		self.grid.Add( self.trigger_value_label, pos=(row,0), \
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		pv_trigger_value_record = "pvin_trig_val.value"

		self.trigger_value_ctrl = MpWx.ValueEntry( self,
				_mpad.server_record, pv_trigger_value_record,
				size=(200,-1) )

		self.grid.Add( self.trigger_value_ctrl, pos=(row,1),
				flag = wx.ALIGN_CENTER_VERTICAL )
		
		#------

		EVT_RESULT( self, self.OnResult )

		#------

	def OnResult( self, event ):

		new_value = self.trigger_pv.get_local()

		new_value = int(new_value)

		print "PV Input OnResult(): new_value =", new_value

		trigger_value = self.trigger_value_ctrl.GetValue()

		trigger_value = int(trigger_value)

		print "PV Input OnResult(): trigger_value =", trigger_value

		self.value.SetLabel( str(new_value) )

		self.value.Enable(True)

		cb_state = self.checkbox.GetValue()

		if ( cb_state == False ):
			print "PV Input OnResult(): Trigger not enabled."
			return

		if ( new_value != trigger_value ):
			print \
	"PV Input OnResult(): New value %d did not match trigger value %d." \
				% ( new_value, trigger_value )
			return

		print "PV Input OnResult(): Triggering the detector."

		biocat_ad_trigger()

		print "PV Input OnResult(): Resetting the input trigger."

		MpCa.poll()

		self.trigger_pv.caput( 0, wait=False )

		MpCa.poll()

		print "PV Input OnResult() complete."

#------
#
# Note: pv_input_value_callback() is invoked in the context of the
#       EPICS subscription callback thread.  However, we can only
#       perform GUI operations from the main thread, so we must
#       post an event to the main thread to tell the input trigger
#       panel to invoke its OnResult() method.
#

def pv_input_value_callback( callback, args ):

	trigger_panel = args

	wx.PostEvent( trigger_panel, ResultEvent(None) )

#------

def pv_input_trigger_name_update( nf, widget, value ):

	print "pv_input_trigger_name_update(): value =", value

	trigger_panel = widget.parent

	biocat_setup_pv_display( trigger_panel, value )

	MpWx._ValueEntry_update( nf, widget, value )

	# print "pv_input_trigger_name_update() complete."

#---------------------------------------------------------------------------
#

class MpadBiocatOutputTriggerPanel( wx.Panel ):
	global pv_output_value_callback
	global pv_output_trigger_name_update
	global EVT_RESULT

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=5, vgap=5 )
		self.SetSizer( self.grid )

		row = 0

		#------

		self.entry_label = wx.StaticText( self, -1, "PV Name:" )

		self.grid.Add( self.entry_label, pos=(row,0),
			flag = wx.ALIGN_CENTER_VERTICAL )
		
		#------

		self.trigger_row = row + 1
		self.trigger_callback_handler = pv_output_value_callback

		pv_name_record = "pvout_trig_name.value"

		self.pv_name = MpWx.ValueEntry( self,
				_mpad.server_record, pv_name_record,
				size=(200,-1),
				function=pv_output_trigger_name_update )

		self.grid.Add( self.pv_name, pos=(row,1),
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		#------

		self.checkbox = wx.CheckBox( self, -1, "Enable" )

		self.grid.Add( self.checkbox, pos=(row,2) )

		row = row + 1

		#------

		self.value_label = wx.StaticText( self, -1, "PV Value" )

		self.grid.Add( self.value_label, pos=(row,0) )

		#------

		self.value = wx.StaticText( self, -1, "--------" )

		self.grid.Add( self.value, pos=(row,1) )

		trigger_pv_name = self.pv_name.GetValue()

		biocat_setup_pv_display( self, trigger_pv_name )

		row = row + 1

		#------

		self.trigger_value_label = wx.StaticText( self, -1,
							"PV Trigger Value:" )

		self.grid.Add( self.trigger_value_label, pos=(row,0),
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		pv_trigger_value_record = "pvout_trig_val.value"

		self.trigger_value_ctrl = MpWx.ValueEntry( self,
				_mpad.server_record, pv_trigger_value_record,
				size=(200,-1) )

		self.grid.Add( self.trigger_value_ctrl, pos=(row,1),
				flag=wx.ALIGN_CENTER_VERTICAL )

		row = row + 1

		#------

		self.trigger_delay_label = wx.StaticText( self, -1,
							"Trigger Delay:" )

		self.grid.Add( self.trigger_delay_label, pos=(row,0),
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		pv_trigger_delay_record = "pvout_delay.value"

		self.trigger_delay_ctrl = MpWx.ValueEntry( self,
				_mpad.server_record, pv_trigger_delay_record,
				size=(200,-1) )

		self.grid.Add( self.trigger_delay_ctrl, pos=(row,1),
				flag=wx.ALIGN_CENTER_VERTICAL )

		#------

		self.trigger_delay_units = wx.StaticText( self, -1, "seconds" )

		self.grid.Add( self.trigger_delay_units, pos=(row,2),
				flag=wx.ALIGN_CENTER_VERTICAL )

		EVT_RESULT( self, self.OnResult )

	def OnResult( self, event ):

		global threading
		global pv_output_timer_callback

		if ( event.data == PVOUT_SET_VALUE ):
			new_value = self.trigger_pv.get_local()

			print "PVOUT_SET_VALUE: new_value =", new_value

			self.value.SetLabel( str(new_value) )

			self.value.Enable(True)

		elif ( event.data == PVOUT_CHECK_IDLE ):
			ad_status = _mpad.ad_record.get_status()

			busy = ad_status & Mp.MXSF_AD_IS_BUSY

			print "PVOUT_CHECK_IDLE: busy =", busy

			# If the area detector is finished, then
			# reset the output trigger PV.

			if ( busy == 0 ):
				print \
				"PVOUT_CHECK_IDLE: Resetting output trigger PV."

				MpCa.poll()
				self.trigger_pv.caput( 0 )
				MpCa.poll()

				return

			# Otherwise, we must reschedule a check of the
			# area detector status for later.

			delay = 1.0		# In seconds

			t = threading.Timer( delay, pv_output_timer_callback, \
							args = [ self ] )

			t.start()

			print "PVOUT_CHECK_IDLE rescheduled."

#------
#
# Note: pv_output_value_callback() is invoked in the context of the
#       EPICS subscription callback thread.  However, we can only
#       perform GUI operations from the main thread, so we must
#       post an event to the main thread to tell the output trigger
#       panel to invoke its OnResult() method.
#

def pv_output_value_callback( callback, args ):

	trigger_panel = args

	wx.PostEvent( trigger_panel, ResultEvent( PVOUT_SET_VALUE ) )

#------

def pv_output_trigger_name_update( nf, widget, value ):

	print "pv_output_trigger_name_update(): value =", value

	trigger_panel = widget.parent

	biocat_setup_pv_display( trigger_panel, value )

	MpWx._ValueEntry_update( nf, widget, value )

	# print "pv_output_trigger_name_update() complete."

#------
#

def pv_output_timer_callback( output_trigger_panel ):
	#
	# Post an event to the output trigger panel telling
	# it to check to see if the area detector is idle.
	# If it is idle, the output trigger will be reset.
	#

	wx.PostEvent( output_trigger_panel, ResultEvent( PVOUT_CHECK_IDLE ) )

#---------------------------------------------------------------------------
#

class MpadBiocatPVTriggerPanel( wx.Panel ):
	global MpadBiocatInputTriggerPanel
	global MpadBiocatOutputTriggerPanel

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.last_poll_time = time.time()

		#----

		self.input_box = wx.StaticBox( self, -1, "PV Input Trigger" )

		self.input_box_sizer = wx.StaticBoxSizer( self.input_box )

		self.input_panel = MpadBiocatInputTriggerPanel( self )

		self.input_box_sizer.Add( self.input_panel, 1, wx.GROW )

		MpCa.poll()
		print "Sleeping ..."
		time.sleep(2)

		#----

		self.output_box = wx.StaticBox( self, -1, "PV Output Trigger" )

		self.output_box_sizer = wx.StaticBoxSizer( self.output_box )

		self.output_panel = MpadBiocatOutputTriggerPanel( self )

		self.output_box_sizer.Add( self.output_panel, 1, wx.GROW )

		MpCa.poll()

		#----

		self.sizer = wx.BoxSizer( wx.HORIZONTAL )

		self.sizer.Add( self.input_box_sizer )
		self.sizer.Add( self.output_box_sizer )

		self.SetSizerAndFit( self.sizer )

		self.Layout()

		self.Bind( wx.EVT_IDLE, self.OnIdle )

	def OnIdle( self, event ):
		# Allow EPICS events to be processed.

		current_time = time.time()

		if ( (current_time - self.last_poll_time) > 5.0 ):
			MpCa.poll()

			self.last_poll_time = current_time


#---------------------------------------------------------------------------
#
# Find the Detector Controls tab and append MpadBiocatPVTriggerPanel
#

detector_controls_tab = _mpad.frame.detector_controls_tab

detector_controls_tab.pv_trigger_panel = \
			MpadBiocatPVTriggerPanel( detector_controls_tab )

detector_controls_tab.box.Add( detector_controls_tab.pv_trigger_panel, \
				border=10, flag=(wx.ALIGN_CENTER|wx.ALL) )

#------

def Biocat_OnStartButton( event ):
	print "Biocat_OnStartButton() invoked."

	biocat_ad_trigger()

	print "Biocat_OnStartButton() complete."

#---------------------------------------------------------------------------
#
# Look in the Right Panel in the Image tab for the Start Sequence button.
#

mpad_right_panel = _mpad.frame.image_tab.right_panel

#
# Replace the existing start button handler.
#

mpad_right_panel.Bind( wx.EVT_BUTTON,
		Biocat_OnStartButton, mpad_right_panel.start_button )

#------

##right_panel_sizer = mpad_right_panel.GetSizer()
##
##right_panel_children = right_panel_sizer.GetChildren()
##
##num_children = len( right_panel_children )
##
##datafile_panel_item_number = -1
##
##for i in range(num_children):
##
##	item = right_panel_children[i]
##
##	window = item.GetWindow()
##
##	if ( type(window) == MpadDatafilePanel ):
##		datafile_panel_item_number = i
##		break
##
##if ( datafile_panel_item_number >= 0 ):
##
##	# Insert the BioCAT trigger panel before the Image Destination panel.
##
##	mpad_right_panel.biocat_trigger = \
##		MpadBiocatInputTriggerPanel( mpad_right_panel )
##
##	right_panel_sizer.Insert( datafile_panel_item_number, \
##				mpad_right_panel.biocat_trigger, \
##				flag=wx.ALIGN_CENTER )
##
##	# An extra set of separators also go before the Image Destination panel.
##
##	mpad_right_panel.biocat_sep1a = wx.StaticLine( mpad_right_panel )
##
##	right_panel_sizer.Insert( datafile_panel_item_number+1, \
##				mpad_right_panel.biocat_sep1a, \
##				border=10, \
##				flag=(wx.TOP | wx.ALIGN_LEFT | wx.EXPAND) )
##
##	mpad_right_panel.biocat_sep1b = wx.StaticLine( mpad_right_panel )
##
##	right_panel_sizer.Insert( datafile_panel_item_number+2, \
##				mpad_right_panel.biocat_sep1b, \
##				border=5, \
##				flag=(wx.BOTTOM | wx.ALIGN_LEFT | wx.EXPAND) )
##

print "***** End of APS-18-ID.py *****"

