#
#----------------------------------------------------------------------------
#

class MpadDetectorControlsTab(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		#
		# Create a Box sizer to contain the controls.
		#

		self.box = wx.BoxSizer( wx.VERTICAL )

		#
		# Attach the sizer to the panel.
		#

		self.SetSizer( self.box )

		#
		# Display the detector name at the top of the panel.
		#

		self.name = wx.StaticText( self, \
				-1, "Aviex PCCD-16080 Controls" )

		self.box.Add( self.name, \
				border=10, flag=(wx.ALIGN_CENTER|wx.ALL) )

		#
		# Display control register values.
		#

		self.control_register_panel = MpadControlRegisterPanel( self )

		self.box.Add( self.control_register_panel, \
				border=10, flag=(wx.ALIGN_CENTER|wx.ALL) )

		#
		# Display Binning
		#

		self.user_register_panel = MpadBinsizePanel( self )

		self.box.Add( self.user_register_panel, \
				border=10, flag=(wx.ALIGN_CENTER|wx.ALL) )

		#
		# Display Correction Flags
		#

		self.correction_flags_panel = MpadCorrectionFlagsPanel( self )

		self.box.Add( self.correction_flags_panel, \
				border=10, flag=(wx.ALIGN_CENTER|wx.ALL) )

#----------------------------------------------------------------------------

class MpadDetectorReadoutBox(MpWx.RadioBox):

	def OnRadioBox( self, event ):

		selection = self.GetSelection()

		if ( selection == 0 ):
			value = 0
		elif ( selection == 1 ):
			value = 2
		elif ( selection == 2 ):
			value = 3
		else:
			print "Illegal detector readout box selection =", \
							selection

		print "MpadDetectorReadoutBox selection = %d, value = %d" \
			% ( selection, value )

		self.nf.put( value )

#----------------------------------------------------------------------------

class MpadControlRegisterPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.box = wx.BoxSizer( wx.HORIZONTAL )

		self.SetSizer( self.box )

		ccd_on_choices = [ 'On', 'Off' ]

		self.ccd_on_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_ccd_on", \
			label="CCD On/Off", \
			choices = ccd_on_choices, \
			reverse_order = True, \
			style=wx.RA_SPECIFY_ROWS )

		self.ccd_on_box.Enable(False)

		self.box.Add( self.ccd_on_box, 1, wx.EXPAND )

		readout_speed_choices = [ 'Low-noise', 'High-speed' ]

		self.readout_speed_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_readout_speed", \
			label="Readout Speed", pos=(160, 10), \
			choices = readout_speed_choices, \
			style=wx.RA_SPECIFY_ROWS )

		self.box.Add( self.readout_speed_box, 1, wx.EXPAND )

		offset_correction_choices = [ 'On', 'Off' ]

		self.automatic_offset_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_offset_correction", \
			label="Automatic Offset Correction", pos=(250, 10), \
			choices = offset_correction_choices, \
			reverse_order = True, \
			style=wx.RA_SPECIFY_ROWS )

		self.box.Add( self.automatic_offset_box, 1, wx.EXPAND )

		exposure_mode_choices = [ 'Internal (software)', 'External' ]

		self.exposure_mode_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_exposure_mode", \
			label="Exposure Trigger", pos=(400, 10), \
			choices = exposure_mode_choices, \
			style=wx.RA_SPECIFY_ROWS )

		self.box.Add( self.exposure_mode_box, 1, wx.EXPAND )

		edge_trigger_choices = [ 'Edge Triggered', 'Duration' ]

		self.edge_trigger_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_edge_triggered", \
			label="External Trigger Type", pos=(520, 10), \
			choices = edge_trigger_choices, \
			style=wx.RA_SPECIFY_ROWS )

		self.box.Add( self.edge_trigger_box, 1, wx.EXPAND )

		_mpad.control_register_nf = mpad_monitor_display(
					"dh_control", self, 1, 0, \
					control_update, None )

	def OnDetectorReadoutBox( self, event ):
		selection = self.detector_readout_box.GetSelection()

		if ( selection == 0 ):
			value = 0
		elif ( selection == 1 ):
			value = 2
		elif ( selection == 2 ):
			value = 3
		else:
			print "Illegal detector_readout_box selection =", \
							selection

		mpad_set_control_register( value, 3, 2 )

#---

def mpad_set_control_register( value, offset, width ):

	mask = 0

	for i in range(width):
		mask <<= 1
		mask |= 1

	mask <<= offset

	new_value = value << offset

	new_value &= mask

	control_register = _mpad.control_register_nf.get()

	new_control_register = control_register & ~mask

	new_control_register |= new_value

	if ( _mpad.debug_serial ):
		print "**********************"
		print "Setting control register to %#x" % new_control_register
		print "**********************"

	_mpad.control_register_nf.put( new_control_register )

#---

def control_update( nf, not_used1, not_used2 ):

	control_panel = _mpad.frame.detector_controls_tab.control_register_panel

	setup_panel = _mpad.frame.setup_parameters_tab.setup_controls_panel

	control_register = nf.get()

	if ( _mpad.debug_serial ):
		print "Received %#x for '%s'" % ( control_register, nf.name )

	#---

	detector_readout_mode = ( control_register >> 3 ) & 0x3

	if ( detector_readout_mode == 0 ):
		control_panel.detector_readout_box.SetSelection(0)

	elif ( detector_readout_mode == 2 ):
		control_panel.detector_readout_box.SetSelection(1)

	elif ( detector_readout_mode == 3 ):
		control_panel.detector_readout_box.SetSelection(2)

	#---

	readout_speed = ( control_register >> 8 ) & 0x1

	control_panel.readout_speed_box.SetSelection( readout_speed )

	#---

	test_mode = ( control_register >> 9 ) & 0x1

	control_panel.test_mode_box.SetSelection( test_mode )

	#---

	automatic_offset_correction = ( control_register >> 7 ) & 0x1

	control_panel.automatic_offset_box.SetSelection( \
					automatic_offset_correction )

	#---

	exposure_mode = ( control_register >> 6 ) & 0x1

	control_panel.exposure_mode_box.SetSelection( exposure_mode )

	#---

	external_trigger = ( control_register >> 5 ) & 0x1

	control_panel.external_trigger_box.SetSelection( external_trigger )

	#---

	linearization = ( control_register >> 2 ) & 0x1

	setup_panel.linearization_box.SetSelection( linearization )

	#---

	offset_correction = ( control_register >> 1 ) & 0x1

	setup_panel.offset_correction_box.SetSelection( offset_correction )

#----------------------------------------------------------------------------

class MpadBinsizePanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=10, vgap=10 )
		self.SetSizer( self.grid )

		#---

		field_name = _mpad.remote_ad_name + ".binsize"

		self.nf = Mp.Net( _mpad.server_record, field_name )

		#---

		allowed_bin_sizes = [ 1, 2, 4, 8, 16, 32, 64, 128 ]

		choice_list = []

		for i in allowed_bin_sizes:
			choice_list.append( str(i) )

		#---

		self.horiz_label = wx.StaticText(self, -1, \
						"Horizontal Binning")

		self.grid.Add( self.horiz_label, pos=(0,0) )

		self.horiz_choice = wx.Choice( self, choices=choice_list )

		self.grid.Add( self.horiz_choice, pos=(1,0) )

		self.horiz_choice.Bind( wx.EVT_CHOICE, self.OnHorizChoice )

		#---

		self.vert_label = wx.StaticText(self, -1, "Vertical Binning")

		self.grid.Add( self.vert_label, pos=(0,1) )

		self.vert_choice = wx.Choice( self, choices=choice_list )

		self.grid.Add( self.vert_choice, pos=(1,1) )

		self.vert_choice.Bind( wx.EVT_CHOICE, self.OnVertChoice )

		# Initialize the choice widgets to their current values.

		current_binsize = self.nf.get()

		self.UpdateChoiceControl( current_binsize )

		# Setup a value changed callback for 'binsize'.

		callback = self.nf.add_callback( Mp.MXCBT_VALUE_CHANGED, \
						_BinsizeUpdateCallback, self )

	def OnHorizChoice( self, event ):

		selection = self.horiz_choice.GetSelection()

		selection_string = self.horiz_choice.GetString( selection )

		selected_horiz_value = int( selection_string )

		old_value = self.nf.get()

		new_value = [ selected_horiz_value, old_value[1] ]

		self.nf.put( new_value )

	def OnVertChoice( self, event ):

		selection = self.vert_choice.GetSelection()

		selection_string = self.vert_choice.GetString( selection )

		selected_vert_value = int( selection_string )

		old_value = self.nf.get()

		new_value = [ old_value[0], selected_vert_value ]

		self.nf.put( new_value )

	def UpdateChoiceControl( self, binsize ):

		horiz_string = str( binsize[0] )

		self.horiz_choice.SetStringSelection( horiz_string )

		vert_string = str( binsize[1] )

		self.vert_choice.SetStringSelection( vert_string )

def _BinsizeUpdateCallback( callback, args ):

	binsize_panel = args

	current_binsize = binsize_panel.nf.get_local_value()

	binsize_panel.UpdateChoiceControl( current_binsize )

#----------------------------------------------------------------------------

class MpadCorrectionFlagsPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=10, vgap=10 )

		self.SetSizer( self.grid )

		self.title = wx.StaticText( self, -1, "Correction Flags" )

		self.grid.Add( self.title, pos=(0,0), span=(1,7), \
			flag=wx.ALIGN_CENTER )

		#---

		self.raw_label = wx.StaticText( self, -1, "Raw Value:" )

		self.grid.Add( self.raw_label, pos=(1,0) )

		corr_flags_name = "%s.correction_flags" % (_mpad.remote_ad_name)

		#---

		self.mask_flag = MpadCorrectionFlagRadioBox( self, "Mask", 0x1 )

		self.grid.Add( self.mask_flag, pos=(1,2) )

		#---

		self.bias_flag = MpadCorrectionFlagRadioBox( self, "Bias", 0x2 )

		self.grid.Add( self.bias_flag, pos=(1,3) )

		#---

		self.dark_current_flag = MpadCorrectionFlagRadioBox( self, \
							"Dark Current", 0x4 )

		self.grid.Add( self.dark_current_flag, pos=(1,4) )

		#---

		self.flood_field_flag = MpadCorrectionFlagRadioBox( self, \
							"Flood Field", 0x8 )

		self.grid.Add( self.flood_field_flag, pos=(1,5) )

		#---

		self.geometrical_flag = MpadCorrectionFlagRadioBox( self, \
							"Geometrical", 0x1000 )

		self.grid.Add( self.geometrical_flag, pos=(1,6) )

		#---
		#
		# Note: This widget must be added last, since creating
		# the widget will eventually result in the callback function
		# _Mpad_flags_value_update() being invoked, which assumes
		# that all of the above widgets have already been created.
		#

		self.raw_value = MpadCorrectionFlagsValue( self, \
				_mpad.server_record, corr_flags_name )

		self.raw_value.SetForegroundColour( "blue" )

		self.grid.Add( self.raw_value, pos=(1,1) )

#--------------------------------------------------------------------------

class MpadCorrectionFlagsValue(MpWx.Value):

	def __init__( self, parent, server_record, corr_flags_name ):

		self.parent = parent

		MpWx.Value.__init__( self, parent, server_record, \
					corr_flags_name, base=16, \
					function=_Mpad_flags_value_update )

def _Mpad_flags_value_update( nf, widget, value ):

	MpWx._Value_update( nf, widget, value )

	flags_panel = widget.parent

	mask_flag = flags_panel.mask_flag

	if ( value & mask_flag.bitmask ):
		mask_flag.SetSelection(0)
	else:
		mask_flag.SetSelection(1)

	bias_flag = flags_panel.bias_flag

	if ( value & bias_flag.bitmask ):
		bias_flag.SetSelection(0)
	else:
		bias_flag.SetSelection(1)

	dark_current_flag = flags_panel.dark_current_flag

	if ( value & dark_current_flag.bitmask ):
		dark_current_flag.SetSelection(0)
	else:
		dark_current_flag.SetSelection(1)

	flood_field_flag = flags_panel.flood_field_flag

	if ( value & flood_field_flag.bitmask ):
		flood_field_flag.SetSelection(0)
	else:
		flood_field_flag.SetSelection(1)

	geometrical_flag = flags_panel.geometrical_flag

	if ( value & geometrical_flag.bitmask ):
		geometrical_flag.SetSelection(0)
	else:
		geometrical_flag.SetSelection(1)

#--------------------------------------------------------------------------

class MpadCorrectionFlagRadioBox(wx.RadioBox):

	def __init__( self, parent, label, bitmask ):

		on_off = ['On','Off']

		wx.RadioBox.__init__( self, parent, \
				label=label, choices=on_off, \
				style=wx.RA_SPECIFY_ROWS )

		self.parent = parent

		self.bitmask = bitmask

		self.Bind( wx.EVT_RADIOBOX, self.OnRadioBox )

	def OnRadioBox( self, event ):

		bit_value = self.GetSelection()

		bit_value = int( bit_value )

		bit_value = 1 - bit_value	;# Put 'On' at the top

		flags_panel = self.parent 

		flags_nf = flags_panel.raw_value.nf

		old_flags = flags_nf.get()

		if ( bit_value ):
			new_flags = old_flags | self.bitmask
		else:
			new_flags = old_flags & (~self.bitmask)

		flags_nf.put( new_flags )

#----------------------------------------------------------------------------

class MpadRegisterControl:

	def __init__( self, parent, field_name, grid, row, label, units, \
			start, step_size, num_steps, scale, offset ):

		field_name = _mpad.remote_ad_name + "." + field_name

		self.parent = parent

		self.start = start
		self.step_size = step_size
		self.num_steps = num_steps

		nf = Mp.Net( _mpad.server_record, field_name )

		read_only = nf.get_attribute( Mp.MXNA_READ_ONLY )

		self.label = wx.StaticText( parent, -1, label )

		grid.Add( self.label, pos=(row,0), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

		if ( read_only ):
			self.value = MpWx.Value( parent, \
					_mpad.server_record, field_name )

			self.value.SetForegroundColour( "blue" )
		else:
			self.value = MpWx.ValueEntry( parent, \
					_mpad.server_record, field_name )

		grid.Add( self.value, pos=(row,1), \
			flag=(wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL) )

		self.units = wx.StaticText( parent, -1, units )

		grid.Add( self.units, pos=(row,2), \
			flag=(wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL) )

#
#----------------------------------------------------------------------------
#

class MpadSetupParametersTab(wx.Panel):

	def __init__( self, parent ):

		wx.Panel.__init__( self, parent )

		self.parent = parent

		#
		# Create a sizer to contain the controls.
		#

		self.box = wx.BoxSizer(wx.VERTICAL)

		#
		# Attach the sizer to the panel.
		#

		self.SetSizer( self.box )

		#
		# Display the detector name at the top of the panel.
		#

		self.name = wx.StaticText( self, \
				-1, "Aviex PCCD-16080 Setup Parameters" )

		self.box.Add( self.name, flag=wx.ALIGN_CENTER )

		#----

		#
		# Display control register values.
		#

#		self.setup_controls_panel = MpadSetupControlsPanel( self )

#		self.box.Add( self.setup_controls_panel, flag=wx.ALIGN_CENTER)

		#
		# Display register settings
		#

		self.setup_register_panel = MpadSetupRegisterPanel( self )

		self.box.Add( self.setup_register_panel, flag=wx.ALIGN_CENTER )

#----------------------------------------------------------------------------

class MpadSetupControlsPanel(wx.Panel):

	def __init__( self, parent ):
		wx.Panel.__init__( self, parent )

		self.parent = parent

#		linearization_choices = [ 'Off', 'On' ]
#
#		self.linearization_box = MpWx.RadioBox( self, \
#			_mpad.server_record, \
#			_mpad.remote_ad_name + ".dh_linearization", \
#			label="Linearization", pos=(10,10), \
#			choices = linearization_choices, \
#			style=wx.RA_SPECIFY_ROWS )
#
		offset_correction_choices = \
		    [ 'Binned Pixel Averaging', 'Un-binned Pixel Averaging' ]

		self.offset_correction_box = MpWx.RadioBox( self, \
			_mpad.server_record, \
			_mpad.remote_ad_name + ".dh_offset_correction", \
			label="Offset Correction Mode", pos=(100,10), \
			choices = offset_correction_choices, \
			style=wx.RA_SPECIFY_ROWS )

#----------------------------------------------------------------------------

class MpadSetupRegisterPanel(wx.Panel):

	def __init__( self, parent ):

		wx.Panel.__init__( self, parent )

		self.parent = parent

		self.grid = wx.GridBagSizer( hgap=10 )

		self.SetSizer( self.grid )

		row = 2

		self.control_ctrl = \
				MpadRegisterControl( self, \
				"dh_control", \
				self.grid, row, \
				"Control register:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.vidbin_ctrl = \
				MpadRegisterControl( self, \
				"dh_vidbin", \
				self.grid, row, \
				"VIDBIN:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.vshbin_ctrl = \
				MpadRegisterControl( self, \
				"dh_vshbin", \
				self.grid, row, \
				"VSHBIN:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.roilines_ctrl = \
				MpadRegisterControl( self, \
				"dh_roilines", \
				self.grid, row, \
				"ROILINES:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.nof_ctrl = \
				MpadRegisterControl( self, \
				"dh_nof", \
				self.grid, row, \
				"NOF:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.vread_ctrl = \
				MpadRegisterControl( self, \
				"dh_vread", \
				self.grid, row, \
				"VREAD:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.vdark_ctrl = \
				MpadRegisterControl( self, \
				"dh_vdark", \
				self.grid, row, \
				"VDARK:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.hlead_ctrl = \
				MpadRegisterControl( self, \
				"dh_hlead", \
				self.grid, row, \
				"HLEAD:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.hpix_ctrl = \
				MpadRegisterControl( self, \
				"dh_hpix", \
				self.grid, row, \
				"HPIX:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.tpre_ctrl = \
				MpadRegisterControl( self, \
				"dh_tpre", \
				self.grid, row, \
				"TPRE:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.shutter_ctrl = \
				MpadRegisterControl( self, \
				"dh_shutter", \
				self.grid, row, \
				"SHUTTER:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.tpost_ctrl = \
				MpadRegisterControl( self, \
				"dh_tpost", \
				self.grid, row, \
				"TPOST:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.hdark_ctrl = \
				MpadRegisterControl( self, \
				"dh_hdark", \
				self.grid, row, \
				"HDARK:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.hbin_ctrl = \
				MpadRegisterControl( self, \
				"dh_hbin", \
				self.grid, row, \
				"HBIN:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.roioffs_ctrl = \
				MpadRegisterControl( self, \
				"dh_roioffs", \
				self.grid, row, \
				"ROIOFFS:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		self.noe_ctrl = \
				MpadRegisterControl( self, \
				"dh_noe", \
				self.grid, row, \
				"NOE:", \
				"", \
				1, 1, 2047, 1, 0 )

		row = row + 1

		#---

		self.xaoffs_ctrl = MpadRegisterControl( self, \
				"dh_xaoffs", \
				self.grid, row, \
				"XA Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.xboffs_ctrl = MpadRegisterControl( self, \
				"dh_xboffs", \
				self.grid, row, \
				"XB Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.xcoffs_ctrl = MpadRegisterControl( self, \
				"dh_xcoffs", \
				self.grid, row, \
				"XC Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.xdoffs_ctrl = MpadRegisterControl( self, \
				"dh_xdoffs", \
				self.grid, row, \
				"XD Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.yaoffs_ctrl = MpadRegisterControl( self, \
				"dh_yaoffs", \
				self.grid, row, \
				"YA Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.yboffs_ctrl = MpadRegisterControl( self, \
				"dh_yboffs", \
				self.grid, row, \
				"YB Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.ycoffs_ctrl = MpadRegisterControl( self, \
				"dh_ycoffs", \
				self.grid, row, \
				"YC Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		self.ydoffs_ctrl = MpadRegisterControl( self, \
				"dh_ydoffs", \
				self.grid, row, \
				"YD Offset:", \
				"", \
				0, 1, 65535, 1, 0 )

		row = row + 1

		#----

		self.fpga_version_name = wx.StaticText( self, -1, \
						"FPGA Version:" )

		self.grid.Add( self.fpga_version_name, \
				pos=(row,0), border=5, \
				flag=(wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL))

		self.fpga_version_value = wx.StaticText( self, -1, \
						"-----" )

		self.fpga_version_value.SetForegroundColour( "blue" )

		self.grid.Add( self.fpga_version_value, \
				pos=(row,1), border=5, \
				flag=(wx.ALIGN_CENTER))

		field_name = "%s.dh_fpga_version" \
					% (_mpad.ad_record.name)

		field_value = Mp.Net(_mpad.server_record, field_name).get()

		field_value = str(field_value)

		self.fpga_version_value.SetLabel(field_value)

